
# q-expansion of newform 1274.2.a.c, downloaded from the LMFDB on 15 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 1274
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[-1], [-1], [2], [0], [1], [1], [-4], [-4], [-5], [6], [-11], [7], [-3], [2], [-3], [-2], [-4], [-5], [1], [-2], [-7], [1], [10], [-6], [-17], [9], [-6], [-10], [2], [5], [-17], [-8], [10], [-20], [-21], [-16], [-5], [12], [-12], [26], [14], [-7], [-12], [24], [19], [-26], [24], [15], [12], [22], [19], [-6], [14], [-9], [28], [-24], [19], [15], [-12], [-2], [-1], [10], [-10], [8], [10], [3], [-31], [-31], [0], [-16], [21], [26], [-10], [-30], [4], [23], [24], [28], [30], [-18], [-5], [-15], [6], [2], [-20], [6], [-20], [28], [-14], [16], [-32], [-24], [-44], [14], [23], [12], [-36], [40], [-17], [10], [26], [31], [19], [-27], [28], [2], [6], [-26], [37], [-28], [-8], [-11], [-24], [0], [-2], [-45], [44], [18], [-6], [44], [30], [-29], [-39], [25], [8], [-14], [45], [-34], [26], [-8], [36], [0], [-25], [-2], [17], [5], [42], [8], [9], [-42], [-56], [-34], [35], [-24], [-50], [15], [10], [28], [33], [56], [25], [-26], [-26], [-16], [-14], [24], [-39], [1], [-52], [12], [20], [30], [-22], [-27], [-6], [-56], [-23], [-41], [-12], [26], [12], [12], [-25], [-10], [10], [-52], [43], [-54], [36], [25], [-16], [-63], [34], [-21], [24], [42], [0], [40], [-20], [-3], [56], [5], [30], [-2], [-16], [-50], [4], [-42], [30], [56], [-11], [-28], [-18], [8], [-28], [-30], [40], [-54], [-22], [67], [-7], [-13], [-47], [-48], [35], [-23], [19], [32], [-38], [-15], [-24], [25], [2], [4], [-51], [-26], [-6], [-51], [54], [39], [-26], [-28], [-12], [-45], [33], [-63], [-49], [-9], [-26], [-60], [-33], [1], [44], [42], [-15], [76], [9], [44], [-67], [29], [1], [-70], [-12], [22], [-67], [0], [58], [-6], [-33], [-45], [-32], [-30], [-4], [51], [41], [-64], [-27], [33], [-41], [-52], [3], [-17], [9], [40], [44], [8], [36], [19], [-38], [-12], [-40], [72], [72], [8], [8], [15], [-39], [-46], [35], [30], [68], [-33], [-54], [-80], [-58], [-30], [-26], [85], [-47], [32], [12], [39], [0], [-6], [12], [-86], [-42], [-46], [-66], [-16], [72], [54], [21], [-2], [68], [-46], [24], [60], [82], [57], [57], [-78], [-17], [56], [-12], [22], [-59], [-66], [60], [36], [-23], [38], [72], [-12], [-73], [-54], [10], [-9], [-54], [-88], [-26], [-3], [54], [-83], [81], [50], [38], [-84], [-69], [-69], [-26], [-2], [9], [-46], [8], [-18], [71], [94], [-45], [-26], [26], [42], [-21], [-75], [47], [-9], [24], [-46], [-19], [-44], [-31], [38], [-34], [44], [26], [-18], [-39], [-35], [-28], [-96], [20], [-44], [-58], [-100], [24], [-52], [72], [77], [48], [12], [84], [10], [55], [52], [-66], [23], [83], [34], [-50], [-10], [8], [-78], [-76], [39], [82], [40], [-53], [-36], [58], [62], [-26], [-24], [30], [88], [-13], [30], [-48], [36], [-23], [6], [22], [-32], [16], [-56], [74], [-68], [-96], [58], [70], [-90]]
